package datavisualizer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import java.util.ArrayList;

import javawrapper.ModelException;

/**
 * This controls the drawing of a histogram for all species at once for a grid 
 * for those grids that display values by species.
 * The label of the data is passed in the constructor.  The histogram is
 * defaulted to 10 divisions equally divided between their minimum and maximum
 * values.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>October 20, 2010: Created (LEM)
 */


public class GridAllSpeciesHistogramDataRequest
    extends DataRequest
    implements ActionListener {

  /**Label root of the piece of data, as it would be passed as a 
   * data member code, with "_[x]" appended (x = species number); so if the 
   * piece of data is named "seeds_x", one per species, then this is "seeds".*/
  private String m_sLabel;
  
  /**Name of grid*/
  private String m_sGridName;

  /**Whether our data member being graphed is a float*/
  private boolean m_bIsFloat;

  /**Whether our data member being graphed is an integer*/
  private boolean m_bIsInt;
  
  /**The dataset that will be graphed*/
  protected ModelHistogramDataset m_oDataset;

  /**Field displaying the number of bins in the histogram*/
  private JTextField m_jNumBins = new JTextField("");

  /**Field displaying the bin size in the histogram*/
  private JTextField m_jBinSize = new JTextField("");

  /**The data member codes for this data member - one per species*/
  private int[] mp_iDataMemberCodes;

  /**The data - kept in an array of vectors - one per species*/
  private ArrayList<ArrayList<Number>> mp_oData;
  
  /**Size of the histogram bins*/
  private float m_fBinSize;

  /**Number of bins in the histogram*/
  private int m_iNumBins;

  /**Whether or not to use a logarithmic axis on the Y*/
  private boolean m_bUseLogarithmicAxis;

  /**Whether or not we have to recalculate the bins on chart update (i.e. we
     are charting an unknown data type without defaults)*/
  private boolean m_bRecalcBinsOnUpdate;

  /**Whether or not to display a total bar in each bin*/
  private boolean m_bShowTotal = true;
  
  /** Formatter for big numbers */
  private java.text.NumberFormat m_jFormat = java.text.NumberFormat.getInstance();

  /** Formatter for very small numbers */
  java.text.DecimalFormat m_jSmallNumsFormat = new java.text.DecimalFormat(
        "0.#E0");

  /**
   * Constructor.
   * @param sLabel Label root of the piece of data, as it would be passed as a 
   * data member code, with "_x" appended; so if the piece of data is named 
   * "seeds_x", one per species, then pass "seeds".
   * @param bIsFloat Whether this is float data.  If this is set to false,
   * integer data is assumed.
   * @param oManager The parent detailed output file manager.
   * @param sGraphName Name of graph string.
   * @param sGridName Name of grid.
   */
  public GridAllSpeciesHistogramDataRequest(String sLabel, boolean bIsFloat,
                              DetailedOutputFileManager oManager,
                              String sGraphName, String sGridName) {
    super(sGraphName, oManager);

    //Default use of logarithmic axis to true
    m_bUseLogarithmicAxis = true;

    //m_fPlotAreaInHectares = oManager.GetPlotArea();

    //Set up flags
    m_sLabel = sLabel;
    m_sGridName = sGridName;
    if (bIsFloat) {
      m_bIsFloat = true;
      m_bIsInt = false;
    }
    else {
      m_bIsFloat = false;
      m_bIsInt = true;
    }
    m_oManager = oManager;
    int iNumSpecies = oManager.GetNumberOfSpecies(),
        i;

    //Declare our data member codes array and initialize our codes to -1
    mp_iDataMemberCodes = new int[iNumSpecies];
    for (i = 0; i < iNumSpecies; i++) {
      mp_iDataMemberCodes[i] = -1;
    }

    //Declare our data vector array
    mp_oData = new ArrayList<ArrayList<Number>>(iNumSpecies);
    for (i = 0; i < iNumSpecies; i++) {
      mp_oData.add(i, new ArrayList<Number>(0));
    }
    
    m_iNumBins = 10;
    m_fBinSize = 0;
    m_bRecalcBinsOnUpdate = true;
    
    //Set up the formatters
    m_jFormat.setMaximumFractionDigits(2);
    m_jFormat.setGroupingUsed(false);
    m_jSmallNumsFormat.setGroupingUsed(false);

  }

  /**
   * Writes the histogram's data to tab-delimited text.
   * @param jOut java.io.FileWriter The file to write to.
   * @throws java.io.IOException if there's a problem writing the file.
   */
  protected void WriteChartDataToFile(java.io.FileWriter jOut) throws java.io.
      IOException {
    try {

      float fValue;
      int i, j, iNumSeries, iNumItems;

      //If there are no series in the dataset, add them now (in case the
      //file was re-parsed while this chart was open)
      if (m_oDataset.getSeriesCount() == 0) {
        AddSeriesToDataset();
        //Adjust values to amounts per hectare
        iNumSeries = m_oDataset.getSeriesCount();
        for (i = 0; i < iNumSeries; i++) {
          iNumItems = m_oDataset.getItemCount(i);
          for (j = 0; j < iNumItems; j++) {
            fValue = m_oDataset.getY(i, j).intValue();
            //fValue /= m_fPlotAreaInHectares;
            m_oDataset.setYValue(i, j, fValue);
          }
        }
      }

      //jOut.write("Values are in number per hectare.\n");

      //Get the number of series - this will be the number of species, with
      //a possible totals series
      iNumSeries = m_oDataset.getSeriesCount();
      if (iNumSeries <= 0) {
        jOut.write("This histogram has no data.\n");
        return;
      }

      //Write out the header columns
      jOut.write("Species");
      for (i = 1; i <= m_iNumBins; i++) {
        jOut.write("\t" + String.valueOf( (i - 1) * m_fBinSize) + " - " +
                   String.valueOf(i * m_fBinSize));
      }
      jOut.write("\n");

      //Write out the series
      for (i = 0; i < iNumSeries; i++) {
        jOut.write((String)m_oDataset.getSeriesKey(i));
        iNumItems = m_oDataset.getItemCount(i);
        for (j = 0; j < iNumItems; j++) {
          jOut.write("\t" + String.valueOf(m_oDataset.getYValue(i, j)));
        }
        jOut.write("\n");
      }
      jOut.write("\n");
    }
    catch (ModelException oErr) {
      ; //do nothing
    }
  }

  /**
   * Performs actions for the controls in the Histogram window.
   * @param oEvent Event triggered.
   */
  public void actionPerformed(ActionEvent oEvent) {
    try {
      if (oEvent.getSource() instanceof JCheckBox) {
        JCheckBox jBox = (JCheckBox) oEvent.getSource();
        if (jBox.getName().equals("log_checkbox")) {
          if (jBox.isSelected() != m_bUseLogarithmicAxis) {
            m_bUseLogarithmicAxis = jBox.isSelected();
            UpdateChart(m_oManager.GetLegend());
          }
        }
        else if (jBox.getName().equals("total_checkbox")) {
          if (jBox.isSelected() != m_bShowTotal) {
            m_bShowTotal = jBox.isSelected();

            //If the dataset already exists, simply calling an update will
            //have no effect; so either remove or add the series, as
            //requested
            if (m_oDataset != null && m_oDataset.getSeriesCount() > 0) {
              if (m_bShowTotal == true) {
                m_oDataset.AddTotalSeries();
              }
              else {
                for (int i = 0; i < m_oDataset.getSeriesCount(); i++) {
                  if (((String)m_oDataset.getSeriesKey(i)).equalsIgnoreCase("total")) {
                    m_oDataset.RemoveSeries(i);
                    break;
                  }
                }
              }
            }
            UpdateChart(m_oManager.GetLegend());
          }
        }        
      }
      else if (oEvent.getActionCommand().equals("UpdateBins")) {

        //Make sure the values in the bins are recognizable, greater-than-zero numbers
        int iNumBins = 0;
        float fBinSize = 0;
        try {
          Integer oNumBins = new Integer(m_jNumBins.getText());
          Float oBinSize = new Float(m_jBinSize.getText());
          iNumBins = oNumBins.intValue();
          fBinSize = oBinSize.floatValue();
        }
        catch (java.lang.NumberFormatException oErr) {
          JOptionPane.showMessageDialog(m_oChartFrame,
                                        "The value for number of bins or bin size is not a recognized number.");
          m_jNumBins.setText(String.valueOf(m_iNumBins));
          m_jBinSize.setText(String.valueOf(m_fBinSize));
          return;
        }

        if (iNumBins <= 0 || fBinSize <= 0) {
          JOptionPane.showMessageDialog(m_oChartFrame,
                                        "The values for number of bins and bin size must be greater than zero.");
          m_jNumBins.setText(String.valueOf(m_iNumBins));
          m_jBinSize.setText(String.valueOf(m_fBinSize));
          return;
        }

        if (iNumBins != m_iNumBins || fBinSize != m_fBinSize) {
          m_iNumBins = iNumBins;
          m_fBinSize = fBinSize;

          //Clear the dataset to force a reconstruction
          m_oDataset = null;
          m_oDataset = new ModelHistogramDataset();

          m_bRecalcBinsOnUpdate = false;
          UpdateChart(m_oManager.GetLegend());
        }
      }
      super.actionPerformed(oEvent);
    }
    catch (javawrapper.ModelException oErr) {
      javawrapper.ErrorGUI oHandler = new javawrapper.ErrorGUI(m_oChartFrame);
      oHandler.WriteErrorMessage(oErr);
    }
  }

  /**
   * Gets the bin size for this data request's histogram.
   * @return The bin size.
   */
  public float GetBinSize() {
    return m_fBinSize;
  }

  /**
   * Gets whether or not to use a logarithmic axis.
   * @return True if a logarithmic axis should be used, or false if a linear
   * axis should be used.
   */
  public boolean GetUseLogarithmicAxis() {
    return m_bUseLogarithmicAxis;
  }
  
  /**
   * Gets the species from a data member label which ends in "_x", where x is
   * the species number.
   * @param sLabel The data member label.
   * @return The species number, or -1 if none exists.
   */
  private int GetSpeciesFromDataMemberLabel(String sLabel) {
    int iPos = sLabel.lastIndexOf("_");
    if (iPos == -1) {
      return -1;
    }
    return new Integer(sLabel.substring(iPos + 1)).intValue();
  }
  
  /**
   * Captures the data member code if we're graphing a float.
   * @param sGridName Name of the grid
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridFloatDataMemberCode(String sGridName, String sLabel,
                                         int iCode) {
    if (m_bIsFloat && sGridName.equals(m_sGridName) && 
        sLabel.startsWith(m_sLabel + "_")) {

      int iSpecies = GetSpeciesFromDataMemberLabel(sLabel);
      if (iSpecies == -1) return;
      //Bingo - it's this data member that we're graphing
      mp_iDataMemberCodes[iSpecies] = iCode; 
    }
  }

  /**
   * Captures the data member code if we're graphing an integer.
   * @param sGridName Name of the grid
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddGridIntDataMemberCode(String sGridName, String sLabel,
                                       int iCode) {
    if (m_bIsInt && sGridName.equals(m_sGridName) && 
        sLabel.startsWith(m_sLabel + "_")) {

      int iSpecies = GetSpeciesFromDataMemberLabel(sLabel);
      if (iSpecies == -1) return;
      //Bingo - it's this data member that we're graphing
      mp_iDataMemberCodes[iSpecies] = iCode; 
    }
  }

 
  /**
   * Creates the histogram using data accumulated thus far.
   * @param oLegend Legend for this chart.
   * @param sChartTitle Title for this chart.
   * @return The new chart window.
   * @throws datavisualizer.ModelException Passing through underlying
   * exceptions.
   */
  ModelInternalFrame DrawChart(Legend oLegend, String sChartTitle) throws
      ModelException {
    int i;

    //Create our new dataset
    m_oDataset = new ModelHistogramDataset();

    AddSeriesToDataset();

    ModelHistogramDataset oAdjustedDataset = UpdateForVisible(oLegend);

    //Adjust values to amounts per hectare
    /*iNumSeries = oAdjustedDataset.getSeriesCount();
    for (i = 0; i < iNumSeries; i++) {
      iNumItems = oAdjustedDataset.getItemCount(i);
      for (j = 0; j < iNumItems; j++) {
        fValue = oAdjustedDataset.getY(i, j).intValue();
        fValue /= m_fPlotAreaInHectares;
        oAdjustedDataset.setYValue(i, j, fValue);
      }
    }*/

    //Make the chart
    m_oChartFrame = DataGrapher.DrawHistogram(oAdjustedDataset, m_sLabel,
                                              "Total Number",
                                              sChartTitle, oLegend, this);

    //Add the extra controls to the top of the histogram window
    JPanel jControls = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.
        LEFT));

    //Bins boxes
    JLabel jTemp = new JLabel("Number of bins:");
    jTemp.setFont(new uihelpers.SortieFont());
    jControls.add(jTemp);
    m_jNumBins.setFont(new uihelpers.SortieFont());
    m_jNumBins.setText(String.valueOf(m_iNumBins));
    m_jNumBins.setPreferredSize(new java.awt.Dimension(50,
        (int) m_jNumBins.getPreferredSize().getHeight()));
    jControls.add(m_jNumBins);

    jTemp = new JLabel("Bin size:");
    jTemp.setFont(new uihelpers.SortieFont());
    jControls.add(jTemp);
    m_jBinSize.setFont(new uihelpers.SortieFont());
    java.text.NumberFormat oFormat = java.text.NumberFormat.getInstance();
    oFormat.setMaximumFractionDigits(2);
    oFormat.setGroupingUsed(false);
    m_jBinSize.setText(oFormat.format(m_fBinSize));
    m_jBinSize.setPreferredSize(new java.awt.Dimension(50,
        (int) m_jBinSize.getPreferredSize().getHeight()));
    jControls.add(m_jBinSize);

    //Button to change
    JButton jButton = new JButton("Change");
    jButton.setFont(new uihelpers.SortieFont());
    jButton.setActionCommand("UpdateBins");
    jButton.addActionListener(this);
    jControls.add(jButton);

    //Checkbox for logarithmic axis
    JCheckBox jCheckBox = new JCheckBox("Use Logarithmic Axis",
                                              GetUseLogarithmicAxis());
    jCheckBox.setFont(new uihelpers.SortieFont());
    jCheckBox.setName("log_checkbox");
    jCheckBox.addActionListener(this);
    jControls.add(jCheckBox);

    //Checkbox for showing total bars
    jCheckBox = new JCheckBox("Show Totals", m_bShowTotal);
    jCheckBox.setFont(new uihelpers.SortieFont());
    jCheckBox.setName("total_checkbox");
    jCheckBox.addActionListener(this);
    jControls.add(jCheckBox);
    
    //Remove the chart, which is the content panel
    org.jfree.chart.ChartPanel jChart = null;
    for (i = 0; i < m_oChartFrame.getContentPane().getComponentCount(); i++) {
      if (m_oChartFrame.getContentPane().getComponent(i) instanceof org.jfree.
          chart.ChartPanel) {
        jChart = (org.jfree.chart.ChartPanel) m_oChartFrame.getContentPane().
            getComponent(i);
      }
    }

    //Recreate the content pane with the controls and the chart
    JPanel jContentPanel = new JPanel(new java.awt.BorderLayout());
    jContentPanel.setLayout(new java.awt.BorderLayout());
    jContentPanel.add(jControls, java.awt.BorderLayout.NORTH);
    jContentPanel.add(jChart, java.awt.BorderLayout.CENTER);

    m_oChartFrame.setContentPane(jContentPanel);

    return m_oChartFrame;

  }

  /**
   * Copies our dataset and includes only species marked as visible in the
   * legend.
   * @param oLegend Legend for this chart.
   * @return The copied dataset.
   * @throws ModelException Passing through an underlying exception.
   */
  protected ModelHistogramDataset UpdateForVisible(Legend oLegend) throws
      ModelException {
    ModelHistogramDataset oNewDataset = (ModelHistogramDataset) m_oDataset.
        clone();
    String sName;
    int i;

    for (i = 0; i < oNewDataset.getSeriesCount(); i++) {
      sName = (String)oNewDataset.getSeriesKey(i);

      if (sName.equalsIgnoreCase("total") == false) {

        //If it's enabled, add the series to the copy
        if (!oLegend.GetIsSpeciesSelected(m_oManager.GetSpeciesCodeFromName(
            sName))) {
          oNewDataset.RemoveSeries(i);
          i--;
        }
      }
    }

    return oNewDataset;
  }

  /**
   * Updates the histogram.
   * @param oLegend Legend for this chart.
   * @throws datavisualizer.ModelException Passing through an underlying
   * exception.
   */
  void UpdateChart(Legend oLegend) throws ModelException {
    
    //If there are no series in the dataset, add them now (in case the
    //file was re-parsed while this chart was open)
    if (m_oDataset.getSeriesCount() == 0) {
      AddSeriesToDataset();
      //Adjust values to amounts per hectare
     /* iNumSeries = m_oDataset.getSeriesCount();
      for (i = 0; i < iNumSeries; i++) {
        iNumItems = m_oDataset.getItemCount(i);
        for (j = 0; j < iNumItems; j++) {
          fValue = m_oDataset.getY(i, j).intValue();
          fValue /= m_fPlotAreaInHectares;
          m_oDataset.setYValue(i, j, fValue);
        }
      }*/
    }

    ModelHistogramDataset oAdjustedDataset = UpdateForVisible(oLegend);

    DataGrapher.UpdateHistogram(oAdjustedDataset, m_oChartFrame, oLegend,
                                this);

    //Update bin information, if we're updating
    if (m_bRecalcBinsOnUpdate) {
      m_jNumBins.setText(String.valueOf(m_iNumBins));
      java.text.NumberFormat oFormat = java.text.NumberFormat.getInstance();
      oFormat.setMaximumFractionDigits(2);
      oFormat.setGroupingUsed(false);
      m_jBinSize.setText(oFormat.format(m_fBinSize));
    }
  }

  /**
   * This will add all series to the dataset.
   * @throws ModelException Passing through from underlying
   */
  protected void AddSeriesToDataset() throws ModelException {
    float[] p_oNewSeries;
    float fMax = 0;
    int i, j;
    
    //If our binsize is 0, figure it out from the data.  Get the maximum value
    //in the dataset.  We'll want to refigure this at every update because we
    //don't know the final limits of whatever we're graphing.
    if (m_bRecalcBinsOnUpdate) {
      //Get the maximum value of all series
      for (i = 0; i < mp_oData.size(); i++) {
        for (j = 0; j < mp_oData.get(i).size(); j++) {
          Number oNumber = (Number) mp_oData.get(i).get(j);
          if (oNumber.floatValue() > fMax) {
            fMax = oNumber.floatValue();
          }
        }
      }
      m_fBinSize = (fMax + 1) / m_iNumBins;
    }

    //Add all accumulated data to the dataset
    for (i = 0; i < mp_oData.size(); i++) {
      p_oNewSeries = new float[mp_oData.get(i).size()];
      
      for (j = 0; j < mp_oData.get(i).size(); j++) {
        Number fVal = (Number) mp_oData.get(i).get(j);
        p_oNewSeries[j] = fVal.floatValue();
      }
      

      if (p_oNewSeries.length > 0) {
        m_oDataset.addSeries(m_oManager.GetSpeciesNameFromCode(i), p_oNewSeries,
                             m_iNumBins, m_fBinSize);
      }
    }

    //If totals are desired, add them
    if (m_bShowTotal) {
      m_oDataset.AddTotalSeries();
    }

  }

  /**
   * This will add the integer data, if the data for this chart is an int that
   * matches the data member code.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param iVal Value.
   */
  public void AddGridIntData(String sGridName, int iX, int iY, int iCode,
      int iVal) {
    int iSpecies;
    if (!m_bIsInt || sGridName.equals(m_sGridName) == false) return; 
    for (iSpecies = 0; iSpecies < mp_iDataMemberCodes.length; iSpecies++) {
      if (mp_iDataMemberCodes[iSpecies] == iCode) {
        //Add this piece of data to our dataset
        mp_oData.get(iSpecies).add(new Integer(iVal));
      }      
    }
  }
  
  /**
   * This will add the float data, if the data for this chart is a float that
   * matches the data member code.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param iVal Value.
   */
  public void AddGridFloatData(String sGridName, int iX, int iY, int iCode,
      float fVal) {
    int iSpecies;
    if (!m_bIsFloat || sGridName.equals(m_sGridName) == false) return; 
    for (iSpecies = 0; iSpecies < mp_iDataMemberCodes.length; iSpecies++) {
      if (mp_iDataMemberCodes[iSpecies] == iCode) {
        //Add this piece of data to our dataset
        mp_oData.get(iSpecies).add(new Float(fVal));
      }      
    }
  }

  /**
   * If our data piece that's being graphed is an integer, this returns true.
   * Otherwise, false.
   * @return Whether this object's data piece is an integer.
   */
  public boolean WantAnyGridInts() {
    return m_bIsInt;
  }

  /**
   * If our data piece that's being graphed is a float, this returns true.
   * Otherwise, false.
   * @return Whether this object's data piece is a float.
   */
  public boolean WantAnyGridFloats() {
    return m_bIsFloat;
  }

  /**
   * This nulls our dataset and recreates it, and clears all of our data
   * vectors.
   */
  void ClearData() {

    m_oDataset = null;
    m_oDataset = new ModelHistogramDataset();

    for (int i = 0; i < mp_oData.size(); i++) {
      mp_oData.get(i).clear();      
    }
  }
}