package javawrapper;

import javax.swing.*;

import uihelpers.SortieFont;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import datavisualizer.XYSimpleCellRenderer;
import datavisualizer.XYZSimpleDataset;

/**
 * Base class for windows for episodic event editing.
 * Copyright: Copyright (c) Charles D. Canham 2003</p>
 * Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 * <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
 */

public class EpisodicEventsEdit
    extends JDialog {
  
  

  /**Copy of the EpisodicEventsWindow object to exchange data with*/
  protected EpisodicEventsWindow m_oEpisodicEventsWindow;

  /**Renderer which displays current cell selections*/
  protected XYSimpleCellRenderer m_oRenderer = new XYSimpleCellRenderer();

  /** Dataset of values that says whether a cell is currently selected (true)
   * or not (false)*/
  protected XYZSimpleDataset m_oDataset;

  /**Panel container for the chart*/
  protected JPanel m_jChartPanel = new JPanel();

  /**ChartPanel displaying the chart*/
  protected org.jfree.chart.ChartPanel m_oChart;

  /**Color for displaying selected cells. Children need to set this value.*/
  protected java.awt.Color m_jColor;

  /**Length of cells in the X direction. Children need to set this value.*/
  protected float m_fLengthXCells;
  /**Length of cells in the Y direction. Children need to set this value.*/
  protected float m_fLengthYCells;

  /**Total number of tree species*/
  protected int m_iNumSpecies;
  /**Number of X cells in the grid.  Children need to set this value.*/
  protected int m_iNumXCells;
  /**Number of Y cells in the grid.  Children need to set this value.*/
  protected int m_iNumYCells;

  /**
   * Constructor.  Creates the window.
   * @param oWindow Object to exchange data with.
   * @param sTitle Window title.
   * @throws ModelException If the window cannot be set up correctly.
   */
  public EpisodicEventsEdit(EpisodicEventsWindow oWindow, String sTitle) throws ModelException {
    super(oWindow, sTitle, true);

    m_oEpisodicEventsWindow = oWindow;

    TreePopulation oPop = m_oEpisodicEventsWindow.GetTreePopulation();
    m_iNumSpecies = oPop.GetNumberOfSpecies();
  }

  /**
   * Sets up the data cell charting.  This creates the cell renderer and the
   * chart, and places the chart in the chart panel.
   * @throws ModelException if the chart cannot be created.
   */
  protected void SetUpCharting() throws ModelException {
    int i, j;

    //Set up our gridded dataset of values - default the white underlayer to
    //all true, the top layer to all false
    m_oDataset = new XYZSimpleDataset(2, m_iNumXCells, m_iNumYCells);
    for (i = 0; i < m_iNumXCells; i++) {
      for (j = 0; j < m_iNumYCells; j++) {
        m_oDataset.mp_bData[0][i][j] = true;
        m_oDataset.mp_bData[1][i][j] = false;
      }
    }

    //Set the underlayer and overlayer colors of the renderer
    m_oRenderer.SetColor(255, 255, 255, 0);
    m_oRenderer.SetColor(m_jColor, 1);
  }

  /**
   * Refreshes the chart to show the data currently held in the dataset.
   * @throws ModelException if the chart cannot be created.
   */
  protected void RefreshChart() throws ModelException {
    int i;

    //Delete all existing components in the chart panel
    for (i = 0; i < m_jChartPanel.getComponentCount(); i++) {
      if (m_jChartPanel.getComponent(i)instanceof org.jfree.chart.
          ChartPanel) {
        m_jChartPanel.remove(i);
      }
    }

    m_oChart = datavisualizer.DataGrapher.MakeOverlaidMap(m_oDataset,
        m_oEpisodicEventsWindow.m_oTreeDataset, "East->", "",
        m_oEpisodicEventsWindow.m_iPlotLengthX,
        m_oEpisodicEventsWindow.m_iPlotLengthY, m_oRenderer,
        m_oEpisodicEventsWindow.m_oTreeRenderer, new java.awt.Dimension(Math.min(600,
                                m_oEpisodicEventsWindow.m_iPlotLengthX * 3), Math.min(600, m_oEpisodicEventsWindow.m_iPlotLengthY * 3)));

    m_jChartPanel.add(m_oChart, BorderLayout.CENTER);
    //For some reason I have to add the next two lines to get the chart to
    //redisplay.
    m_jChartPanel.validate();
    m_jChartPanel.repaint();

    //Set the mouse listener
    EpisodicEventsMouseListener oListener = new EpisodicEventsMouseListener(this);
    m_oChart.addChartMouseListener(oListener);
    m_oChart.addMouseListener(oListener);
    m_oChart.addMouseMotionListener(oListener);
  }

  /**
   * Creates a tree legend panel.
   * @param oPop TreePopulation Tree population object, for querying for
   * existence of trees.
   * @return JPanel Panel with tree legend on it.
   */
  protected JPanel MakeTreeLegendPanel(TreePopulation oPop) {
    int i;

    JPanel jLegendPanel = new JPanel(new GridLayout(0, 1));
    if (oPop.GetTrees().size() == 0) {
      JLabel jTitle = new JLabel("No tree map trees to display.");
      jTitle.setFont(new SortieFont());
      jLegendPanel.add(jTitle);
    }
    for (i = 0; i < m_iNumSpecies; i++) {
      JPanel jTemp = new JPanel(new FlowLayout(FlowLayout.LEFT));
      java.awt.Color jSpeciesColor = (java.awt.Color) m_oEpisodicEventsWindow.
          m_oTreeRenderer.getSeriesPaint(i);
      JButton jColorButton = new JButton(new ModelIcon(15, 15, ModelIcon.RECTANGLE, jSpeciesColor));
      jColorButton.setContentAreaFilled(false);
      jColorButton.setPreferredSize(new java.awt.Dimension(15, 15));
      JLabel jTempSpLabel = new JLabel(oPop.GetSpeciesNameFromCode(i).replace(
          '_', ' '));
      jTempSpLabel.setFont(new SortieFont());
      jTemp.add(jColorButton);
      jTemp.add(jTempSpLabel);
      jLegendPanel.add(jTemp);
    }
    return jLegendPanel;
  }

}
