package javawrapper;

/**
 * This is the organizer class for the dead remover behavor.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>November 18, 2010: Made dead remover automatic and added the mortality 
 * reporter behavior; changed the name of this to "mortality utilities" (LEM)
 */

public class MortalityUtilitiesBehaviors
    extends BehaviorTypeBase {
  
  /**Probability of dead trees becoming standing snags - natural mortality */
  protected ModelVector mp_fTreeRemNatMortSnagStandProb = new ModelVector(
      "Probability of Trees Becoming Snags - Natural Mortality (0-1)",
      "mo_treeRemNatMortSnagsStanding", "mo_trnmssVal",
      0, ModelVector.FLOAT);
  
  /**Probability of dead trees becoming standing snags - storm mortality */
  protected ModelVector mp_fTreeRemStormMortSnagStandProb = new ModelVector(
      "Probability of Trees Becoming Snags - Storm Mortality (0-1)",
      "mo_treeRemStormMortSnagsStanding", "mo_trsmssVal",
      0, ModelVector.FLOAT);
  
  /**Probability of dead trees becoming standing snags - disease mortality */
  protected ModelVector mp_fTreeRemDiseaseMortSnagStandProb = new ModelVector(
      "Probability of Trees Becoming Snags - Disease Mortality (0-1)",
      "mo_treeRemDiseaseMortSnagsStanding", "mo_trdmssVal",
      0, ModelVector.FLOAT);
  
  /**Probability of dead trees becoming standing snags - insect mortality */
  protected ModelVector mp_fTreeRemInsectMortSnagStandProb = new ModelVector(
      "Probability of Trees Becoming Snags - Insect Mortality (0-1)",
      "mo_treeRemInsectMortSnagsStanding", "mo_trimssVal",
      0, ModelVector.FLOAT);
  
  /**Probability of dead trees becoming standing snags - fire mortality */
  protected ModelVector mp_fTreeRemFireMortSnagStandProb = new ModelVector(
      "Probability of Trees Becoming Snags - Fire Mortality (0-1)",
      "mo_treeRemFireMortSnagsStanding", "mo_trfmssVal",
      0, ModelVector.FLOAT);

  /**
   * Constructor
   * @param oManager GUIManager object.
   */
  public MortalityUtilitiesBehaviors(GUIManager oManager) {
    super(oManager, "Mortality utilities", "mortalityUtility");

    //Set up behavior list
    mp_oChildBehaviors = new Behavior[1];

    //Add a behavior for dead removal
    mp_oChildBehaviors[0] = new Behavior("removedead", "Dead tree remover",
                                         "removedead", (float)2, (float)1);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.STUMP, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SNAG, true);
    /*mp_oChildBehaviors[0].AddRequiredData(mp_fTreeRemNatMortSnagStandProb);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeRemStormMortSnagStandProb);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeRemDiseaseMortSnagStandProb);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeRemInsectMortSnagStandProb);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTreeRemFireMortSnagStandProb);*/
    
    //Add a behavior for mortality reporting 
    /*mp_oChildBehaviors[1] = new Behavior("Mortality reporter", 
        "Mortality reporter", "Mortality reporter", (float)1, (float)1);
    mp_oChildBehaviors[1].m_bMustHaveTrees = false; */
    
    mp_oAllData.add(mp_fTreeRemNatMortSnagStandProb);
    mp_oAllData.add(mp_fTreeRemStormMortSnagStandProb);
    mp_oAllData.add(mp_fTreeRemDiseaseMortSnagStandProb);
    mp_oAllData.add(mp_fTreeRemInsectMortSnagStandProb);
    mp_oAllData.add(mp_fTreeRemFireMortSnagStandProb);
  }


  /**
   * Sets up the grid for the mortality reporter behavior, and the defaults for
   * standing snag probabilities.  
   * @param oPop Not used.
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    /*DataMember[] p_oDataMembers;
    Plot p_oPlot = m_oManager.GetPlot();
    Grid oNewGrid;
    int iNumSpecies = oPop.GetNumberOfSpecies(),
        iMortCode,
        iNumFloats, iStart = 0, i;
    
    //There are more reason codes than this that are supported in the C++
    //version of this grid, but those mortality types are not yet fully 
    //implemented.  So I am not going to include them to avoid confusing the 
    //users.  
    iNumFloats = (3 * 3 * iNumSpecies) + //density, 3=tree types, 3=reason codes
                 (2 * iNumSpecies) + //snag BA and density
                 (2 * 3 * iNumSpecies); //BA, 2=tree types, 3=mort reason
    
    //Mortality reporter grid 
    String sGridName = "Dead Trees";
    p_oDataMembers = new DataMember[iNumFloats];    
    
    //Seedlings, number, natural mortality 
    iMortCode = MortalityBehaviors.natural; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Seedlings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Natural Mort", 
          "num_seedlings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Seedlings, number, disease mortality 
    iMortCode = MortalityBehaviors.disease; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Seedlings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Disease Mort", 
          "num_seedlings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Seedlings, number, storm  mortality 
    iMortCode = MortalityBehaviors.storm; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Seedlings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Storm Mort", 
          "num_seedlings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Saplings, number, natural mortality 
    iMortCode = MortalityBehaviors.natural; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Saplings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Natural Mort", 
          "num_saplings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Saplings, number, disease mortality 
    iMortCode = MortalityBehaviors.disease; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Saplings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Disease Mort", 
          "num_saplings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Saplings, number, storm  mortality 
    iMortCode = MortalityBehaviors.storm; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Saplings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Storm Mort", 
          "num_saplings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Adults, number, natural mortality 
    iMortCode = MortalityBehaviors.natural; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Adults " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Natural Mort", 
          "num_adults_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Adults, number, disease mortality 
    iMortCode = MortalityBehaviors.disease; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Adults " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Disease Mort", 
          "num_adults_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Adults, number, storm  mortality 
    iMortCode = MortalityBehaviors.storm; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Adults " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Storm Mort", 
          "num_adults_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Saplings, basal area, natural mortality 
    iMortCode = MortalityBehaviors.natural; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Saplings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Natural Mort", 
          "ba_saplings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Saplings, basal area, disease mortality 
    iMortCode = MortalityBehaviors.disease; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Saplings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Disease Mort", 
          "ba_saplings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Saplings, basal area, storm  mortality 
    iMortCode = MortalityBehaviors.storm; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Saplings " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Storm Mort", 
          "ba_saplings_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Adults, basal area, natural mortality 
    iMortCode = MortalityBehaviors.natural; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Adults " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Natural Mort", 
          "ba_adults_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Adults, basal area, disease mortality 
    iMortCode = MortalityBehaviors.disease; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Adults " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Disease Mort", 
          "ba_adults_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Adults, basal area, storm  mortality 
    iMortCode = MortalityBehaviors.storm; 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Adults " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') +
          " Storm Mort", 
          "ba_adults_" + iMortCode + "_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Snags, number 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("Num Snags " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') + " Removed", 
          "num_snags_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    //Snags, basal area 
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[iStart] = new DataMember("BA Snags " + 
          oPop.GetSpeciesNameFromCode(i).replace('_', ' ') + " Removed", 
          "ba_snags_" + i, DataMember.FLOAT);
        iStart++;
    }
    
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 
        p_oPlot.GetPlotXLength(), p_oPlot.GetPlotYLength());
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Mortality reporter").AddGrid(oNewGrid);
    
    //**********
    // Snag probabilities defaults
    //**********
    if (mp_fTreeRemDiseaseMortSnagStandProb.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fTreeRemDiseaseMortSnagStandProb.GetValue().add(new Float(1.0));
      }
    }
    
    if (mp_fTreeRemFireMortSnagStandProb.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fTreeRemFireMortSnagStandProb.GetValue().add(new Float(1.0));
      }
    }
    
    if (mp_fTreeRemInsectMortSnagStandProb.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fTreeRemInsectMortSnagStandProb.GetValue().add(new Float(1.0));
      }
    }
    
    if (mp_fTreeRemNatMortSnagStandProb.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fTreeRemNatMortSnagStandProb.GetValue().add(new Float(1.0));
      }
    }
    
    if (mp_fTreeRemStormMortSnagStandProb.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fTreeRemStormMortSnagStandProb.GetValue().add(new Float(1.0));
      }
    }*/
  };

  /**
  * Makes sure that this is enabled if there is a mortality behavior 
  * enabled.
  * @param oPop Not used.
  * @throws ModelException if the dead tree remover is enabled but no
  * mortality behavior is.
  */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    MortalityBehaviors oMort = m_oManager.GetMortalityBehaviors(); 
    Behavior oRemBeh = GetBehaviorByKey("removedead");
    Behavior[] p_oMorts = oMort.GetAllBehaviors(); 
    int i, j; 
    
    oRemBeh.ClearSpeciesTypeCombos(); 
    for(i = 0; i < p_oMorts.length; i++) {
      if (p_oMorts[i].m_bIsEnabled) {
        for (j = 0; j < p_oMorts[i].GetNumberOfCombos(); j++) {
          oRemBeh.AddSpeciesTypeCombo((SpeciesTypeCombo)p_oMorts[i].GetSpeciesTypeCombo(j).clone()); 
        }   
      }
    }
  }
}
