package javawrapper;


import java.io.FileWriter;
import java.io.File;

/**
 * Tests the MortalityBehaviors class.
 * Copyright: Copyright (c) Charles D. Canham 2003</p>
 * Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestMortalityBehaviors
    extends ModelTestCase {

  /**
   * Makes sure that nci lambda values are managed correctly when species
   * change.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Species 1"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                    0.6640108, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                    0.71, 0.01);

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex + 1);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                    0.00442797, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                    0.12, 0.01);

      //Change the species
      String[] sNewSpecies = new String[] {
          "Species 2",
          "Species 3",
          "Species 4"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oMort = oManager.GetMortalityBehaviors();

      //Count the total number of NCIs
      int iCount = 0;
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iCount++;
        }
      }
      assertEquals(3, iCount);

      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                    0.12, 0.01);

      iIndex++;
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Species 3") &&
                 -1 != oVector.GetDescriptor().indexOf("NCI Lambda"));

      iIndex++;
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Species 4") &&
                 -1 != oVector.GetDescriptor().indexOf("NCI Lambda"));

      iIndex++;
      if (iIndex < oMort.mp_oAllData.size()) {
        oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
        assertTrue( -1 == oVector.GetDescriptor().indexOf("NCI Lambda"));
      }

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Makes sure that nci lambda values are managed correctly when species
   * are copied.
   */
  public void testCopySpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLFileForSpeciesCopy();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }

      //Verify how the data was read in before species copy
      Behavior oBeh = oMort.GetBehaviorByKey("NCI Mortality");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(4, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }

      Float fVal;

      //Max survival
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(0);
      assertEquals(0.1, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(1);
      assertEquals(0.3, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(2);
      assertEquals(0.5, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(3);
      assertEquals(0.7, fVal.floatValue(), 0.001);

      //NCI Alpha
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);

      //NCI Beta
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);

      //Lambdas
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.33, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.31, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.87, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);

      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Western_Redcedar",
                                                 "Black_Cottonwood");

      oBeh = oMort.GetBehaviorByKey("NCI Mortality");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(5, oBeh.GetNumberOfCombos());
      for (i = 0; i < oBeh.GetNumberOfCombos() - 1; i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(3, oCombo.GetType());
      assertEquals(7, oCombo.GetSpecies());

      //Max survival
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(0);
      assertEquals(0.1, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(1);
      assertEquals(0.3, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(2);
      assertEquals(0.5, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(3);
      assertEquals(0.7, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(7);
      assertEquals(0.3, fVal.floatValue(), 0.001);

      //NCI Alpha
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(7);
      assertEquals(1.86, fVal.floatValue(), 0.001);

      //NCI Beta
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(7);
      assertEquals(0.46, fVal.floatValue(), 0.001);

      //Lambdas
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(7);
      assertEquals(0.19, fVal.floatValue(), 0.001);

      System.out.println("Test copy species was successful.");
    }
    catch (ModelException oErr) {
      fail("Test copy species failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
  
  /**
   * Makes sure that nci lambda values are managed correctly when species
   * are copied for species whose names are very short ("A").
   */
  public void testCopySpecies2() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLFileForSpeciesCopy();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
      
      
      //Create a new species
      String[] sNewSpecies = new String[] {
          "Western Hemlock",
          "Western Redcedar",
          "Amabilis Fir",
          "Subalpine Fir",
          "Hybrid Spruce",
          "Lodgepole Pine",
          "Trembling Aspen",
          "Black Cottonwood",
          "Paper Birch",
          "A"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Western_Redcedar", "A");
      
      
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }

      //Verify the data
      Behavior oBeh = oMort.GetBehaviorByKey("NCI Mortality");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(5, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo;
      for (i = 0; i < oBeh.GetNumberOfCombos()-1; i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(3, oCombo.GetType());
      assertEquals(9, oCombo.GetSpecies());

      Float fVal;

      //Max survival
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(0);
      assertEquals(0.1, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(1);
      assertEquals(0.3, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(2);
      assertEquals(0.5, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(3);
      assertEquals(0.7, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCIMaxProbSurvival.GetValue().elementAt(9);
      assertEquals(0.3, fVal.floatValue(), 0.001);

      //NCI Alpha
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDBHEffect.GetValue().elementAt(9);
      assertEquals(1.86, fVal.floatValue(), 0.001);

      //NCI Beta
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      fVal = (Float) oMort.mp_fNCINeighDistanceEffect.GetValue().elementAt(9);
      assertEquals(0.46, fVal.floatValue(), 0.001);

      //Lambdas
      for (i = 0; i < oMort.mp_oAllData.size(); i++) {
        oData = (ModelData) oMort.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.33, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.31, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.87, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oMort.mp_oAllData.elementAt(iIndex);
      assertTrue( -1 != oVector.GetDescriptor().indexOf("A "));
      fVal = (Float) oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float) oVector.GetValue().elementAt(9);
      assertEquals(0.49, fVal.floatValue(), 0.001);
    
      System.out.println("Test copy species was successful.");
    }
    catch (ModelException oErr) {
      fail("Test copy species failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      TreePopulation oPop;
      MortalityBehaviors oMortality = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no mortality behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      assertEquals(null, oMortality.FormatDataForDisplay(oPop));

      //Case:  only GMF mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("gmfmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Light-Dependent Mortality",
          "Mortality at Zero Growth"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only BC mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("bcmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Light-Dependent Mortality",
          "Mortality at Zero Growth"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only adult self-thinning is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("adultselfthin");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Self-Thinning Slope",
          "Maximum DBH for Adult Self-Thinning",
          "Adult Self-Thinning Intercept"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only juvenile self-thinning is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("juvselfthin");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SAPLING,
          oPop));
      p_sExpected = new String[] {
          "Juvenile Self-Thinning Slope",
          "Juvenile Self-Thinning Intercept"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only senescence is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("senescence");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Senescence Mortality Alpha",
          "Senescence Mortality Beta",
          "DBH of Maximum Senescence Mortality Rate, as an integer in cm",
          "DBH at Onset of Senescence, in cm"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only adult stochastic mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("adultstochmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Background Mortality Rate"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only juvenile stochastic mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("juvstochmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SAPLING,
          oPop));
      p_sExpected = new String[] {
          "Juvenile Background Mortality Rate"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only weibull snag mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SNAG,
          oPop));
      p_sExpected = new String[] {
          "Weibull Annual \"a\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Upper DBH of Snag Size Class 1",
          "Weibull Upper DBH of Snag Size Class 2"
      };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only nci mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("NCI Mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Max Radius of Crowding Neighbors, in m",
          "NCI Neighbor DBH Effect (alpha)",
          "NCI Neighbor Distance Effect (beta)",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Max Survival Probability (0-1)",
          "NCI Size Effect Mode, in cm",
          "NCI Size Effect Variance, in cm",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Storm Effect - Medium Damage (0-1)",
          "NCI Storm Effect - Complete Damage (0-1)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Minimum Neighbor DBH, in cm",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "Species 1 NCI Lambda Neighbors"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only competition mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("competition mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Competition Mortality Shape Parameter (Z)",
          "Competition Mortality Maximum Parameter (max)"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only growth and resource mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("resource mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Growth/Resource - Scaling Factor (rho)",
          "Growth/Resource - Function Mode (mu)",
          "Growth/Resource - Survival Increase with Growth (delta)",
          "Growth/Resource - Low-Growth Survival Parameter (sigma)"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only density self-thinning mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("density self-thinning mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Density Self-Thinning Asymptote (A)",
          "Density Self-Thinning Diameter Effect (C)",
          "Density Self-Thinning Density Effect (S)",
          "Density Self-Thinning Minimum Density for Mortality (#/ha)",
          "Density Self-Thinning Neighborhood Radius, in m"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only logistic bi-level mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("logistic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Logistic Bi-Level - Low-Light \"a\"",
          "Logistic Bi-Level - Low-Light \"b\"",
          "Logistic Bi-Level - High-Light \"a\"",
          "Logistic Bi-Level - High-Light \"b\"",
          "Logistic Bi-Level - High-Light Mortality Threshold (0-100)"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only stochastic bi-level mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("stochastic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Stochastic Bi-Level - Low-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Threshold"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only height-gli weibull mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("height-gli weibull mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Height-GLI Weibull - a",
          "Height-GLI Weibull - b",
          "Height-GLI Weibull - c",
          "Height-GLI Weibull - d",
          "Height-GLI Weibull - Max Mortality (0 - 1)",
          "Height-GLI Weibull - Browsed a",
          "Height-GLI Weibull - Browsed b",
          "Height-GLI Weibull - Browsed c",
          "Height-GLI Weibull - Browsed d",
          "Height-GLI Weibull - Browsed Max Mortality (0 - 1)"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only exponential growth and resource based mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("exp resource mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Exponential Growth-Resource - a",
          "Exponential Growth-Resource - b",
          "Exponential Growth-Resource - c",
          "Exponential Growth-Resource - d"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only aggregated mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("aggregated mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Aggregated Mortality Return Interval (years)",
          "Aggregated Mortality Annual Kill Amount (0-1)",
          "Aggregated Mortality Number of Trees To Aggregate",
          "Aggregated Mortality Clumping Parameter",
          "Aggregated Mortality Clump Size"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only browsed stochastic mortality is enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("browsed stochastic mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Juvenile Background Mortality Rate",
          "Browsed Juvenile Background Mortality Rate"};
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only post harvest skidding mortality is enabled. 
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("post harvest skidding mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Post Harvest Skid Mort - Pre-Harvest Background Mort Rate", 
          "Post Harvest Skid Mort - Windthrow Harvest Basic Prob", 
          "Post Harvest Skid Mort - Snag Recruitment Basic Prob", 
          "Post Harvest Skid Mort - Windthrow Size Effect", 
          "Post Harvest Skid Mort - Windthrow Intensity Effect", 
          "Post Harvest Skid Mort - Snag Recruitment Skidding Effect", 
          "Post Harvest Skid Mort - Windthrow Crowding Effect", 
          "Post Harvest Skid Mort - Snag Recruitment Crowding Effect", 
          "Post Harvest Skid Mort - Windthrow Harvest Rate Param", 
          "Post Harvest Skid Mort - Snag Recruitment Rate Param", 
          "Post Harvest Skid Mort - Windthrow Background Prob", 
          "Post Harvest Skid Mort - Snag Recruitment Background Prob", 
          "Post Harvest Skid Mort - Crowding Effect Radius", 
          };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only gompertz density self-thinning mortality is enabled. 
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("Gompertz Density Self Thinning");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Gompertz Density Self Thinning - G", 
          "Gompertz Density Self Thinning - H", 
          "Gompertz Density Self Thinning - I", 
          "Gompertz Density Self Thinning - Min Neighbor Height (m)", 
          "Gompertz Density Self Thinning - Neighbor Search Radius (m)", 
          };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only Weibull climate survival  is enabled. 
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("weibull climate survival");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Weibull Climate Survival - Adult Competition Effect \"C\"",
          "Weibull Climate Survival - Juvenile Competition Effect \"C\"",
          "Weibull Climate Survival - Adult Competition Effect \"D\"",
          "Weibull Climate Survival - Juvenile Competition Effect \"D\"",
          "Weibull Climate Survival - Adult Competition Gamma",
          "Weibull Climate Survival - Juvenile Competition Gamma",
          "Weibull Climate Survival - Max Neighbor Search Radius (m)",
          "Weibull Climate Survival - Minimum Neighbor DBH (cm)",
          "Weibull Climate Survival - Adult Size Effect X0",
          "Weibull Climate Survival - Juvenile Size Effect X0",
          "Weibull Climate Survival - Adult Size Effect Xb",
          "Weibull Climate Survival - Juvenile Size Effect Xb",
          "Weibull Climate Survival - Size Effect Minimum DBH",
          "Weibull Climate Survival - Adult Precip Effect \"A\"",
          "Weibull Climate Survival - Juvenile Precip Effect \"A\"",
          "Weibull Climate Survival - Adult Precip Effect \"B\"",
          "Weibull Climate Survival - Juvenile Precip Effect \"B\"",
          "Weibull Climate Survival - Adult Precip Effect \"C\"",
          "Weibull Climate Survival - Juvenile Precip Effect \"C\"",
          "Weibull Climate Survival - Adult Temp Effect \"A\"",
          "Weibull Climate Survival - Juvenile Temp Effect \"A\"",
          "Weibull Climate Survival - Adult Temp Effect \"B\"",
          "Weibull Climate Survival - Juvenile Temp Effect \"B\"",
          "Weibull Climate Survival - Adult Temp Effect \"C\"",
          "Weibull Climate Survival - Juvenile Temp Effect \"C\"",
          "Weibull Climate Survival - Adult Max Survival Prob (0-1)",
          "Weibull Climate Survival - Juv Max Survival Prob (0-1)",
          
          };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);
      
      
      //Case: only temperature dependent neighborhood survival is enabled. 
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("temp dependent neigh surv");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Temp Dependent Neighborhood Surv - A",
          "Temp Dependent Neighborhood Surv - B",
          "Temp Dependent Neighborhood Surv - M",
          "Temp Dependent Neighborhood Surv - N",
          "Temp Dependent Neighborhood Surv - Neigh Search Radius (m)",           
      };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected); 
      
      //Case: only suppression duration mortality is enabled. 
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("suppression duration mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Suppression Duration Mortality - Max Mortality Rate (0-1)",
          "Suppression Duration Mortality - X0",
          "Suppression Duration Mortality - Xb"
      };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only insect infestation mortality is enabled. 
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      oBeh = oMortality.GetBehaviorByKey("insect mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Insect Mortality - Intercept", 
          "Insect Mortality - Max Mortality Rate (0-1)", 
          "Insect Mortality - X0",
          "Insect Mortality - Xb"           
      };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected); 

      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoMortalityXMLFile());
      oPop = oManager.GetTreePopulation();
      oMortality = oManager.GetMortalityBehaviors();
      int i;
      for (i = 0; i < oMortality.mp_oChildBehaviors.length; i++) {
        oMortality.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (oMortality.mp_oChildBehaviors[i].GetKey().equals("juvstochmort") ||
            oMortality.mp_oChildBehaviors[i].GetKey().equals("juvselfthin") ||
            oMortality.mp_oChildBehaviors[i].GetKey().equals(
            "density self-thinning mortality")) {
          oMortality.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(
              0, TreePopulation.SAPLING, oPop));
        }
        else if (oMortality.mp_oChildBehaviors[i].GetKey().equals(
            "weibull snag mortality")) {
          oMortality.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(
              0, TreePopulation.SNAG, oPop));
        }
        else {
          oMortality.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(0, 3, oPop));
        }
      }
      p_sExpected = new String[] {
          "Light-Dependent Mortality",
          "Mortality at Zero Growth",
          "Juvenile Self-Thinning Slope",
          "Juvenile Self-Thinning Intercept",
          "Adult Self-Thinning Slope",
          "Maximum DBH for Adult Self-Thinning",
          "Adult Self-Thinning Intercept",
          "Senescence Mortality Alpha",
          "Senescence Mortality Beta",
          "DBH of Maximum Senescence Mortality Rate, as an integer in cm",
          "DBH at Onset of Senescence, in cm",
          "Adult Background Mortality Rate",
          "Juvenile Background Mortality Rate",
          "Weibull Annual \"a\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Upper DBH of Snag Size Class 1",
          "Weibull Upper DBH of Snag Size Class 2",
          "Density Self-Thinning Asymptote (A)",
          "Density Self-Thinning Diameter Effect (C)",
          "Density Self-Thinning Density Effect (S)",
          "Density Self-Thinning Minimum Density for Mortality (#/ha)",
          "Density Self-Thinning Neighborhood Radius, in m",
          "NCI Max Radius of Crowding Neighbors, in m",
          "NCI Neighbor DBH Effect (alpha)",
          "NCI Neighbor Distance Effect (beta)",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Max Survival Probability (0-1)",
          "NCI Size Effect Mode, in cm",
          "NCI Size Effect Variance, in cm",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Storm Effect - Medium Damage (0-1)",
          "NCI Storm Effect - Complete Damage (0-1)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Minimum Neighbor DBH, in cm",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "Species 1 NCI Lambda Neighbors",
          "Growth/Resource - Scaling Factor (rho)",
          "Growth/Resource - Function Mode (mu)",
          "Growth/Resource - Survival Increase with Growth (delta)",
          "Growth/Resource - Low-Growth Survival Parameter (sigma)",
          "Competition Mortality Shape Parameter (Z)",
          "Competition Mortality Maximum Parameter (max)",
          "Logistic Bi-Level - Low-Light \"a\"",
          "Logistic Bi-Level - Low-Light \"b\"",
          "Logistic Bi-Level - High-Light \"a\"",
          "Logistic Bi-Level - High-Light \"b\"",
          "Logistic Bi-Level - High-Light Mortality Threshold (0-100)",
          "Stochastic Bi-Level - Low-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Threshold",
          "Height-GLI Weibull - a",
          "Height-GLI Weibull - b",
          "Height-GLI Weibull - c",
          "Height-GLI Weibull - d",
          "Height-GLI Weibull - Max Mortality (0 - 1)",
          "Height-GLI Weibull - Browsed a",
          "Height-GLI Weibull - Browsed b",
          "Height-GLI Weibull - Browsed c",
          "Height-GLI Weibull - Browsed d",
          "Height-GLI Weibull - Browsed Max Mortality (0 - 1)",
          "Aggregated Mortality Return Interval (years)",
          "Aggregated Mortality Annual Kill Amount (0-1)",
          "Aggregated Mortality Number of Trees To Aggregate",
          "Aggregated Mortality Clumping Parameter",
          "Aggregated Mortality Clump Size",
          "Post Harvest Skid Mort - Pre-Harvest Background Mort Rate", 
          "Post Harvest Skid Mort - Windthrow Harvest Basic Prob", 
          "Post Harvest Skid Mort - Snag Recruitment Basic Prob", 
          "Post Harvest Skid Mort - Windthrow Size Effect", 
          "Post Harvest Skid Mort - Windthrow Intensity Effect", 
          "Post Harvest Skid Mort - Snag Recruitment Skidding Effect", 
          "Post Harvest Skid Mort - Windthrow Crowding Effect", 
          "Post Harvest Skid Mort - Snag Recruitment Crowding Effect", 
          "Post Harvest Skid Mort - Windthrow Harvest Rate Param", 
          "Post Harvest Skid Mort - Snag Recruitment Rate Param", 
          "Post Harvest Skid Mort - Windthrow Background Prob", 
          "Post Harvest Skid Mort - Snag Recruitment Background Prob", 
          "Post Harvest Skid Mort - Crowding Effect Radius",
          "Exponential Growth-Resource - a",
          "Exponential Growth-Resource - b",
          "Exponential Growth-Resource - c",
          "Exponential Growth-Resource - d",
          "Browsed Juvenile Background Mortality Rate",
          "Gompertz Density Self Thinning - G", 
          "Gompertz Density Self Thinning - H", 
          "Gompertz Density Self Thinning - I", 
          "Gompertz Density Self Thinning - Min Neighbor Height (m)", 
          "Gompertz Density Self Thinning - Neighbor Search Radius (m)",
          "Weibull Climate Survival - Adult Competition Effect \"C\"",
          "Weibull Climate Survival - Juvenile Competition Effect \"C\"",
          "Weibull Climate Survival - Adult Competition Effect \"D\"",
          "Weibull Climate Survival - Juvenile Competition Effect \"D\"",
          "Weibull Climate Survival - Adult Competition Gamma",
          "Weibull Climate Survival - Juvenile Competition Gamma",
          "Weibull Climate Survival - Max Neighbor Search Radius (m)",
          "Weibull Climate Survival - Minimum Neighbor DBH (cm)",
          "Weibull Climate Survival - Adult Size Effect X0",
          "Weibull Climate Survival - Juvenile Size Effect X0",
          "Weibull Climate Survival - Adult Size Effect Xb",
          "Weibull Climate Survival - Juvenile Size Effect Xb",
          "Weibull Climate Survival - Size Effect Minimum DBH",
          "Weibull Climate Survival - Adult Precip Effect \"A\"",
          "Weibull Climate Survival - Juvenile Precip Effect \"A\"",
          "Weibull Climate Survival - Adult Precip Effect \"B\"",
          "Weibull Climate Survival - Juvenile Precip Effect \"B\"",
          "Weibull Climate Survival - Adult Precip Effect \"C\"",
          "Weibull Climate Survival - Juvenile Precip Effect \"C\"",
          "Weibull Climate Survival - Adult Temp Effect \"A\"",
          "Weibull Climate Survival - Juvenile Temp Effect \"A\"",
          "Weibull Climate Survival - Adult Temp Effect \"B\"",
          "Weibull Climate Survival - Juvenile Temp Effect \"B\"",
          "Weibull Climate Survival - Adult Temp Effect \"C\"",
          "Weibull Climate Survival - Juvenile Temp Effect \"C\"",
          "Weibull Climate Survival - Adult Max Survival Prob (0-1)",
          "Weibull Climate Survival - Juv Max Survival Prob (0-1)", 
          "Temp Dependent Neighborhood Surv - A",
          "Temp Dependent Neighborhood Surv - B",
          "Temp Dependent Neighborhood Surv - M",
          "Temp Dependent Neighborhood Surv - N",
          "Temp Dependent Neighborhood Surv - Neigh Search Radius (m)",
          "Insect Mortality - Intercept", 
          "Insect Mortality - Max Mortality Rate (0-1)", 
          "Insect Mortality - X0",
          "Insect Mortality - Xb",
          "Suppression Duration Mortality - Max Mortality Rate (0-1)",
          "Suppression Duration Mortality - X0",
          "Suppression Duration Mortality - Xb"
      };
      TestTable(oMortality.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for mortality.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests backwards compatibility for Weibull snag mortality behavior
   * and NCI mortality.
   */
 /* public void testBackwardsCompatibility() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //***************
       // Weibull snag mortality
       //***************
        //Valid file 1
     oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
      float fExpected;
      int iNumSpecies = oManager.GetTreePopulation().GetNumberOfSpecies(), i;
      fExpected = 12;
      for (i = 0; i < iNumSpecies; i++) {
        assertEquals(fExpected,
                     ( (Float) oMort.mp_fSnagSizeClass1Dbh.GetValue().
                      elementAt(i)).floatValue(), 0.01);
      }
      fExpected = 30;
      for (i = 0; i < iNumSpecies; i++) {
        assertEquals(fExpected,
                     ( (Float) oMort.mp_fSnagSizeClass2Dbh.GetValue().
                      elementAt(i)).floatValue(), 0.01);
      }
      fExpected = 100;
      assertEquals(100, oMort.m_iDbhAtAsymptoticMaximumMortality.GetValue());
      new File(sFileName).delete();

      //***************
       // NCI mortality
       //***************

    }
    catch (ModelException oErr) {
      fail("Mortality validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

  }*/

  /**
   * Tests ValidateData().
   */
  public void testValidateData() {
    GUIManager oManager = null;
    TreePopulation oPop;
    String sFileName = null;
    try {

      try {
        oManager = new GUIManager(null);

        /********************
          NCI mortality testing
         *********************/
        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        new File(sFileName).delete();
      }
      catch (ModelException oErr) {
        fail("Mortality validation failed with message " + oErr.getMessage());
      }

      //Exception processing - error file 1
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile1();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 1.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 2
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile2();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 2.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 3
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile3();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 3.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 4
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile4();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 4.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 5
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile5();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 5.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 6
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile6();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 6.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 7
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile7();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 7.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 8
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile8();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 8.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 9
      try {
        oManager.ClearCurrentData();
        sFileName = TestNCIMortality.WriteXMLErrorFile9();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 9.");
      }
      catch (ModelException oErr) {
        ;
      }

      /********************
        Density self-thinning mortality testing
       *********************/
      try {
        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = TestDensitySelfThinningMortality.WriteValidFile();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        new File(sFileName).delete();

      }
      catch (ModelException oErr) {
        fail("Mortality validation failed with message " + oErr.getMessage());
      }

      //Exception processing - error file 1
      try {
        oManager.ClearCurrentData();
        sFileName = TestDensitySelfThinningMortality.WriteErrorFile1();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 1.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 2
      try {
        oManager.ClearCurrentData();
        sFileName = TestDensitySelfThinningMortality.WriteErrorFile2();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 2.");
      }
      catch (ModelException oErr) {
        ;
      }
      catch (java.io.IOException oE) {
        fail("Caught IOException.  Message: " + oE.getMessage());
      }

      //Exception processing - error file 3
      try {
        oManager.ClearCurrentData();
        sFileName = TestDensitySelfThinningMortality.WriteErrorFile3();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 3.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 4
      try {
        oManager.ClearCurrentData();
        sFileName = TestDensitySelfThinningMortality.WriteErrorFile4();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 4.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Exception processing - error file 5
      try {
        oManager.ClearCurrentData();
        sFileName = TestDensitySelfThinningMortality.WriteErrorFile5();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetMortalityBehaviors().ValidateData(oManager.
            GetTreePopulation());
        fail("Didn't catch error in error file 5.");
      }
      catch (ModelException oErr) {
        ;
      }

      try {
        //Light threshold not between 0 and 100 for linear bi-level mortality
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oPop = oManager.GetTreePopulation();
        oManager.InputXMLParameterFile(sFileName);
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("logistic bi-level mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fLogBiLevHiLiteThreshold.GetValue().add(new Float( -20));
        oMort.mp_fLogBiLevHiLiteThreshold.GetValue().add(new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch bad light threshold values.");

      }
      catch (ModelException oErr) {
        ;
      }

      //size class 1 a values = 0 for weibull mortality
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fSnag1WeibullA.GetValue().add(new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad weibull snag \"a\" values.");

      }
      catch (ModelException oErr) {
        ;
      }

      //size class 2 a values = 0 for weibull mortality
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fSnag2WeibullA.GetValue().add(new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad weibull snag \"a\" values.");
      }
      catch (ModelException oErr) {
        ;
      }

      //size class 3 a values = 0 for weibull mortality
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fSnag3WeibullA.GetValue().add(new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad weibull snag \"a\" values.");

      }
      catch (ModelException oErr) {
        ;
      }

      //size class 1 a values < 0 plus non-whole-number b for weibull mortality
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fSnag1WeibullA.GetValue().add(new Float( -3));
        //Whole number - should be OK
        oMort.mp_fSnag1WeibullB.GetValue().add(new Float(4));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to non-whole-number
        oMort.mp_fSnag1WeibullB.GetValue().add(0, new Float(0.25));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad weibull snag \"a\" values.");

      }
      catch (ModelException oErr) {
        ;
      }

      //size class 2 a values < 0 plus non-whole-number b for weibull mortality
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fSnag2WeibullA.GetValue().add(new Float( -3));
        //Whole number - should be OK
        oMort.mp_fSnag2WeibullB.GetValue().add(new Float(4));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to non-whole-number
        oMort.mp_fSnag2WeibullB.GetValue().add(0, new Float(0.25));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch bad weibull snag \"a\" values.");
      }
      catch (ModelException oErr) {
        ;
      }

      //size class 3 a values < 0 plus non-whole-number b for weibull mortality
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fSnag3WeibullA.GetValue().add(new Float( -3));
        //Whole number - should be OK
        oMort.mp_fSnag3WeibullB.GetValue().add(new Float(4));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to non-whole-number
        oMort.mp_fSnag3WeibullB.GetValue().add(0, new Float(0.25));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad weibull snag \"a\" values.");

      }
      catch (ModelException oErr) {
        ;
      }

      /********************
       Stochastic bi-level mortality testing
       *********************/
      //High-light threshold value not between 0 and 100
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
        oBeh.m_bIsEnabled = true;
        //Should be OK
        oMort.mp_fStochBiLevHiLiteThreshold.GetValue().add(new Float(50));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fStochBiLevHiLiteThreshold.GetValue().add(0, new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad stochastic bi-level threshold values.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Low-light mortality probability not between 0 and 1
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
        oBeh.m_bIsEnabled = true;
        //Should be OK
        oMort.mp_fStochBiLevLoLiteMortProb.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fStochBiLevLoLiteMortProb.GetValue().add(0, new Float(2));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch bad stochastic bi-level low-light probability values.");
      }
      catch (ModelException oErr) {
        ;
      }

      //High-light mortality probability not between 0 and 1
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
        oBeh.m_bIsEnabled = true;
        //Should be OK
        oMort.mp_fStochBiLevHiLiteMortProb.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fStochBiLevHiLiteMortProb.GetValue().add(0, new Float(2));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch bad stochastic bi-level low-light probability values.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Storm light not enabled
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oPop = oManager.GetTreePopulation();
        oManager.InputXMLParameterFile(sFileName);
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch storm light not enabled.");
      }
      catch (ModelException oErr) {
        ;
      }

      /********************
        Height gli weibull mortality testing
       *********************/
      //Max mortality not between 0 and 1
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

        Behavior oBeh = oMort.GetBehaviorByKey("height-gli weibull mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        //Should be OK
        oMort.mp_fHeightGLIWeibMaxMort.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fHeightGLIWeibMaxMort.GetValue().add(0, new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch bad max mortality values.");
      }
      catch (ModelException oErr) {
        ;
      }

      /********************
       Aggregated mortality testing
       *********************/
      //Amount to cut not a proportion
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("aggregated mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        //Should be OK
        oMort.m_fAggMortPropToKill.SetValue( (float) 0.5);
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.m_fAggMortPropToKill.SetValue( (float) 1.5);
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad aggregated mortality amount to cut.");
      }
      catch (ModelException oErr) {
        ;
      }

      //Return interval is negative
      try {
        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oPop = oManager.GetTreePopulation();
        oManager.InputXMLParameterFile(sFileName);
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("aggregated mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        //Should be OK
        oMort.m_fAggMortEpisodeReturnInterval.SetValue(5);
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.m_fAggMortEpisodeReturnInterval.SetValue((float)- 1.5);
        oMort.ValidateData(oManager.GetTreePopulation());
        fail(
            "Mortality validation failed to catch bad aggregated mortality return interval.");
      }
      catch (ModelException oErr) {
        ;
      }

      /********************
       Browsed stochastic mortality testing
       *********************/
      //Max mortality not between 0 and 1
      try {
        oManager = new GUIManager(null);

        //Case: unbrowsed rate not between 0 and 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("browsed stochastic mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        //Should be OK
        oMort.mp_fJuvenileRandomMortality.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fJuvenileRandomMortality.GetValue().add(0, new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch unbrowsed mortality rates.");
      }
      catch (ModelException oErr) {
        ;
      }

      try {
        //Case: unbrowsed rate not between 0 and 1
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("browsed stochastic mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        //Should be OK
        oMort.mp_fBrowsedJuvenileRandomMortality.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fBrowsedJuvenileRandomMortality.GetValue().add(0,
            new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch unbrowsed mortality rates.");
      }
      catch (ModelException oErr) {
        ;
      }
      
      try {
        //Case: gompertz density self-thinning radius negative
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("Gompertz Density Self Thinning");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.m_fGompertzSearchRadius.SetValue(-9);
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch gompertz density self-thinning radius negative.");
      }
      catch (ModelException oErr) {
        ;
      }
      
      try {
        //Case: gompertz density self-thinning minimum height negative
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("Gompertz Density Self Thinning");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        //Should be OK
        oMort.mp_fGompertzMinHeight.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        //Set to bad value
        oMort.mp_fGompertzMinHeight.GetValue().add(0,
            new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch gompertz density self-thinning minimum height negative.");
      }
      catch (ModelException oErr) {
        ;
      }
      
      try {
        //Case: Weibull climate survival adult survival not a proportion 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimAdultMaxSurv.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimAdultMaxSurv.GetValue().add(0, new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival adult survival not a proportion.");
      }
      catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival juvenile survival not a proportion 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimJuvMaxSurv.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimJuvMaxSurv.GetValue().add(0, new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival juvenile survival not a proportion.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival crowding radius not negative 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimMaxCrowdingRadius.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimMaxCrowdingRadius.GetValue().add(0, new Float( -20));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival negative max crowding radius.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival adult precipitation A = 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimAdultPrecipA.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimAdultPrecipA.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival adult precipitation A = 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival juvenile precipitation A = 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimJuvenilePrecipA.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimJuvenilePrecipA.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival juvenile precipitation A = 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival adult temperature  A = 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimAdultTempA.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimAdultTempA.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival adult temperature A = 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival juvenile temperature  A = 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimJuvenileTempA.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimJuvenileTempA.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival juvenile temperature A = 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival adult X0 < 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimAdultSizeX0.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimAdultSizeX0.GetValue().add(0, new Float(-10));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival adult X0 <= 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival juvenile X0 < 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimJuvenileSizeX0.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimJuvenileSizeX0.GetValue().add(0, new Float(-10));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival juvenile X0 <= 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival adult Xb = 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimAdultSizeXb.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimAdultSizeXb.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival adult Xb = 0.");
      } catch (ModelException oErr) {;}
      
      try {
        //Case: Weibull climate survival juvenile Xb = 0 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("weibull climate survival");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fWeibClimJuvenileSizeXb.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fWeibClimJuvenileSizeXb.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch weibull climate survival juvenile Xb = 0.");
      } catch (ModelException oErr) {;}

      //******************
      // Competition mortality testing
      //*******************
      try {
        //NCI not enabled for all species 
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("competition mortality");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
        oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("nci");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch NCI not enabled.");
      } catch (ModelException oErr) {;}
      
      //******************
      // Temperature dependent neighborhood survival testing
      //*******************
      try {
        //N equals zero  
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("temp dependent neigh surv");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.mp_fTempDepSurvN.GetValue().add(new Float(0.5));
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.mp_fTempDepSurvN.GetValue().add(0, new Float(0));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch temperature dependent neighborhood survival N equals zero.");
      } catch (ModelException oErr) {;}
      
      try {
        //Neighbor search radius is negative   
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("temp dependent neigh surv");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.m_fTempDepSurvSearchRadius.SetValue(new Float(10)); 
        oMort.ValidateData(oManager.GetTreePopulation());
        oMort.m_fTempDepSurvSearchRadius.SetValue(new Float(-10));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch temperature dependent neighborhood survival negative neighbor search radius.");
      } catch (ModelException oErr) {;}
      
      //******************
      // Insect infestation mortality
      //*******************
      try {
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("insect mortality");
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch lack of insect infestation.");
      } catch (ModelException oErr) {;}
      try {
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("insect mortality");
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
        oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Insect Infestation");
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oMort.ValidateData(oManager.GetTreePopulation());
        fail("Mortality validation failed to catch lack of insect infestation.");
      } catch (ModelException oErr) {;}
      try {
        oManager.ClearCurrentData();
        sFileName = WriteXMLFile2();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
        Behavior oBeh = oMort.GetBehaviorByKey("insect mortality");
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
        oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Insect Infestation");
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
        oMort.ValidateData(oManager.GetTreePopulation());
      } catch (ModelException oErr) {throw(new java.io.IOException(oErr.getMessage()));}
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("Mortality ValidateData testing succeeded.");
  }

  /**
   * Writes test XML file.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename of file just written.
   */
  protected String WriteXMLFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>weibull snag mortality</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Snag\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>senescence</behaviorName> ");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>removedead</behaviorName>");
    oOut.write("<version>2.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Snag\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_snag1WeibullA>");
    oOut.write("<mo_s1waVal species=\"Species_3\">0.025</mo_s1waVal>");
    oOut.write("<mo_s1waVal species=\"Species_1\">0.113</mo_s1waVal>");
    oOut.write("</mo_snag1WeibullA>");
    oOut.write("<mo_snag2WeibullA>");
    oOut.write("<mo_s2waVal species=\"Species_3\">0.09</mo_s2waVal>");
    oOut.write("<mo_s2waVal species=\"Species_1\">0.0113</mo_s2waVal>");
    oOut.write("</mo_snag2WeibullA>");
    oOut.write("<mo_snag3WeibullA>");
    oOut.write("<mo_s3waVal species=\"Species_3\">0.0204</mo_s3waVal>");
    oOut.write("<mo_s3waVal species=\"Species_1\">0.1</mo_s3waVal>");
    oOut.write("</mo_snag3WeibullA>");
    oOut.write("<mo_snag1WeibullB>");
    oOut.write("<mo_s1wbVal species=\"Species_3\">3</mo_s1wbVal>");
    oOut.write("<mo_s1wbVal species=\"Species_1\">3.049</mo_s1wbVal>");
    oOut.write("</mo_snag1WeibullB>");
    oOut.write("<mo_snag2WeibullB>");
    oOut.write("<mo_s2wbVal species=\"Species_3\">2</mo_s2wbVal>");
    oOut.write("<mo_s2wbVal species=\"Species_1\">2.2</mo_s2wbVal>");
    oOut.write("</mo_snag2WeibullB>");
    oOut.write("<mo_snag3WeibullB>");
    oOut.write("<mo_s3wbVal species=\"Species_3\">2.5</mo_s3wbVal>");
    oOut.write("<mo_s3wbVal species=\"Species_1\">2.3</mo_s3wbVal>");
    oOut.write("</mo_snag3WeibullB>");
    oOut.write("<mo_randomMortAlpha>");
    oOut.write("<mo_rmaVal species=\"Species_1\">-20.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_2\">-20.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_3\">-20.0</mo_rmaVal>");
    oOut.write("</mo_randomMortAlpha>");
    oOut.write("<mo_randomMortBeta>");
    oOut.write("<mo_rmbVal species=\"Species_1\">0.135</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_2\">0.135</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_3\">0.135</mo_rmbVal>");
    oOut.write("</mo_randomMortBeta>");
    oOut.write("<mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_daoosVal species=\"Species_1\">100.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_2\">100.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_3\">100.0</mo_daoosVal>");
    oOut.write("</mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_dbhAtAsympMaxMort>100</mo_dbhAtAsympMaxMort>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with no mortality settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNoMortalityXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}

/**
 * This class is to help group test code.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class TestDensitySelfThinningMortality {
  /**
   * Writes a file of valid values for density self-thinning.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteValidFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the timestep length is greater than 1.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<rt_timestep>0</rt_timestep>");
    oOut.write("<randomSeed>0</randomSeed>");
    oOut.write("<yearsPerTimestep>3.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>100.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the neighborhood radius is less than or equal to 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">-9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the minimum density is less than 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">-20</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the self-thinning asymptote is less than or equal
   * to 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">-0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the density effect is less than or equal to 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">-0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes stuff common to all parameter files in this class.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteCommonStuff(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<rt_timestep>0</rt_timestep>");
    oOut.write("<randomSeed>0</randomSeed>");
    oOut.write("<yearsPerTimestep>1.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>100.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }
}

/**
 * This class is to help group test code.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class TestNCIMortality {

  /**
   * Writes an error condition where Xb for a species = 0.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    oOut.write("<mo_prnciSizeEffectVar>");
    oOut.write("<mo_psevVal species=\"Species_1\">0</mo_psevVal>");
    oOut.write("<mo_psevVal species=\"Species_2\">5</mo_psevVal>");
    oOut.write("</mo_prnciSizeEffectVar>");
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where X0 for a species = 0.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    oOut.write("<mo_prnciSizeEffectMode>");
    oOut.write("<mo_psemVal species=\"Species_1\">0</mo_psemVal>");
    oOut.write("<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>");
    oOut.write("</mo_prnciSizeEffectMode>");
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where med. damage eta for a species is not a
   * proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    oOut.write("<mo_prnciNeighStormEffMedDmg>");
    oOut.write("<mo_pnsemdVal species=\"Species_1\">-0.33</mo_pnsemdVal>");
    oOut.write("<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>");
    oOut.write("</mo_prnciNeighStormEffMedDmg>");
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where full damage eta for a species is not a
   * proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    oOut.write("<mo_prnciNeighStormEffFullDmg>");
    oOut.write("<mo_pnsefdVal species=\"Species_1\">-0.54</mo_pnsefdVal>");
    oOut.write("<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>");
    oOut.write("</mo_prnciNeighStormEffFullDmg>");
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where med. damage storm Effect for a species is
   * not a proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    oOut.write("<mo_prnciStormEffMedDmg>");
    oOut.write("<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>");
    oOut.write("<mo_psemdVal species=\"Species_2\">1.9</mo_psemdVal>");
    oOut.write("</mo_prnciStormEffMedDmg>");
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where full damage storm Effect for a species is
   * not a proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    oOut.write("<mo_prnciStormEffFullDmg>");
    oOut.write("<mo_psefdVal species=\"Species_1\">-0.42</mo_psefdVal>");
    oOut.write("<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>");
    oOut.write("</mo_prnciStormEffFullDmg>");
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes NCI mortality file with a min neighbor DBH < 0.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteXMLErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    oOut.write("<mo_nciMinNeighborDBH>");
    oOut.write("<mo_nmndVal species=\"Species_1\">10</mo_nmndVal>");
    oOut.write("<mo_nmndVal species=\"Species_2\">-10</mo_nmndVal>");
    oOut.write("</mo_nciMinNeighborDBH>");
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes the beginning portion of a test parameter file.
   * @param oOut FileWriter File stream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteValidateFilePart1(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  /**
   * Writes a valid NCI mortality file.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  public static String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file to test backwards compatibility of NCI mortality.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 16, 2005: Created (LEM)
   */
  public static String WriteXMLCompatFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>puerto rico nci mortality</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where max radius of neighbor Effects is < 0.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_prnciMaxCrowdingRadius>");
    oOut.write("<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Species_2\">-10</mo_pmcrVal>");
    oOut.write("</mo_prnciMaxCrowdingRadius>");
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where max probability of survival for a species
   * is not between 0 and 1.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    oOut.write("<mo_prnciMaxPotentialSurvival>");
    oOut.write("<mo_pmpsVal species=\"Species_1\">15.31</mo_pmpsVal>");
    oOut.write("<mo_pmpsVal species=\"Species_2\">-15.31</mo_pmpsVal>");
    oOut.write("</mo_prnciMaxPotentialSurvival>");
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes NCI mortality behavior list.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteBehaviorList(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>NCI Mortality</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes NCI max crowding radius parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCIMaxCrowdingRadius(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciMaxCrowdingRadius>");
    oOut.write("<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>");
    oOut.write("</mo_prnciMaxCrowdingRadius>");
  }

  /**
   * Writes NCI minimum neighbor DBH parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCIMinNeighborDBH(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_nciMinNeighborDBH>");
    oOut.write("<mo_nmndVal species=\"Species_1\">10</mo_nmndVal>");
    oOut.write("<mo_nmndVal species=\"Species_2\">10</mo_nmndVal>");
    oOut.write("</mo_nciMinNeighborDBH>");
  }

  /**
   * Writes NCI neighbor DBH effect parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCINeighborDBHEffect(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciNeighDBHEff>");
    oOut.write("<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>");
    oOut.write("<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>");
    oOut.write("</mo_prnciNeighDBHEff>");
  }

  /**
   * Writes NCI neighbor distance effect parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCINeighborDistEffect(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<mo_prnciNeighDistEff>");
    oOut.write("<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>");
    oOut.write("<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>");
    oOut.write("</mo_prnciNeighDistEff>");
  }

  /**
   * Writes NCI max survival probability parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCIMaxProbSurvival(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciMaxPotentialSurvival>");
    oOut.write("<mo_pmpgVal species=\"Species_1\">0.5</mo_pmpgVal>");
    oOut.write("<mo_pmpgVal species=\"Species_2\">0.7</mo_pmpgVal>");
    oOut.write("</mo_prnciMaxPotentialSurvival>");
  }

  /**
   * Writes NCI size sensitivity to NCI parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCISizeSensNCI(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSizeSensNCI>");
    oOut.write("<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>");
    oOut.write("<mo_pssnVal species=\"Species_2\">0.36</mo_pssnVal>");
    oOut.write("</mo_prnciSizeSensNCI>");
  }

  /**
   * Writes NCI size effect mode parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCISizeEffectMode(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSizeEffectMode>");
    oOut.write("<mo_psemVal species=\"Species_1\">34.24638994</mo_psemVal>");
    oOut.write("<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>");
    oOut.write("</mo_prnciSizeEffectMode>");
  }

  /**
   * Writes NCI size effect variance parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCISizeEffectVar(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSizeEffectVar>");
    oOut.write("<mo_psevVal species=\"Species_1\">5</mo_psevVal>");
    oOut.write("<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>");
    oOut.write("</mo_prnciSizeEffectVar>");
  }

  /**
   * Writes NCI slope parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCISlope(FileWriter oOut) throws java.io.IOException {
    oOut.write("<mo_prnciNCISlope>");
    oOut.write("<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>");
    oOut.write("<mo_pnslVal species=\"Species_2\">4.57</mo_pnslVal>");
    oOut.write("</mo_prnciNCISlope>");
  }

  /**
   * Writes NCI steepness parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCISteepness(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciNCISteepness>");
    oOut.write("<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>");
    oOut.write("<mo_pnstVal species=\"Species_2\">1.26</mo_pnstVal>");
    oOut.write("</mo_prnciNCISteepness>");
  }

  /**
   * Writes NCI storm effect medium damage parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCIStormEffMedDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciStormEffMedDmg>");
    oOut.write("<mo_psemdVal species=\"Species_1\">0.89</mo_psemdVal>");
    oOut.write("<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>");
    oOut.write("</mo_prnciStormEffMedDmg>");
  }

  /**
   * Writes NCI storm effect complete damage parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCIStormEffFullDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciStormEffFullDmg>");
    oOut.write("<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>");
    oOut.write("<mo_psefdVal species=\"Species_2\">0.34</mo_psefdVal>");
    oOut.write("</mo_prnciStormEffFullDmg>");
  }

  /**
   * Writes NCI neighbor storm effect medium damage parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCINeighStormEffMedDmg(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<mo_prnciNeighStormEffMedDmg>");
    oOut.write("<mo_pnsemdVal species=\"Species_1\">0.83</mo_pnsemdVal>");
    oOut.write("<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>");
    oOut.write("</mo_prnciNeighStormEffMedDmg>");
  }

  /**
   * Writes NCI neighbor storm effect complete damage parameter.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCINeighStormEffFullDmg(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<mo_prnciNeighStormEffFullDmg>");
    oOut.write("<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>");
    oOut.write("<mo_pnsefdVal species=\"Species_2\">0.27</mo_pnsefdVal>");
    oOut.write("</mo_prnciNeighStormEffFullDmg>");
  }

  /**
   * Writes NCI lambdas parameters.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteNCILambdas(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSpecies_1NeighborLambda>");
    oOut.write("<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>");
    oOut.write("<mo_plVal species=\"Species_2\">0.71</mo_plVal>");
    oOut.write("</mo_prnciSpecies_1NeighborLambda>");
    oOut.write("<mo_prnciSpecies_2NeighborLambda>");
    oOut.write("<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>");
    oOut.write("<mo_plVal species=\"Species_2\">0.12</mo_plVal>");
    oOut.write("</mo_prnciSpecies_2NeighborLambda>");
  }

  /**
   * Writes the end portion of valid parameter files.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteValidFileEnd(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
  }

  protected static String WriteXMLFileForSpeciesCopy() throws java.io.
      IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Western_Hemlock\" />");
    oOut.write("<tr_species speciesName=\"Western_Redcedar\" />");
    oOut.write("<tr_species speciesName=\"Amabilis_Fir\" />");
    oOut.write("<tr_species speciesName=\"Subalpine_Fir\" />");
    oOut.write("<tr_species speciesName=\"Hybrid_Spruce\" />");
    oOut.write("<tr_species speciesName=\"Lodgepole_Pine\" />");
    oOut.write("<tr_species speciesName=\"Trembling_Aspen\" />");
    oOut.write("<tr_species speciesName=\"Black_Cottonwood\" />");
    oOut.write("<tr_species speciesName=\"Paper_Birch\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>NCI Mortality</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Western_Hemlock\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Western_Redcedar\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Amabilis_Fir\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Subalpine_Fir\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_prnciMaxPotentialSurvival>");
    oOut.write("<mo_pmpgVal species=\"Western_Hemlock\">0.1</mo_pmpgVal>");
    oOut.write("<mo_pmpgVal species=\"Western_Redcedar\">0.3</mo_pmpgVal>");
    oOut.write("<mo_pmpgVal species=\"Amabilis_Fir\">0.5</mo_pmpgVal>");
    oOut.write("<mo_pmpgVal species=\"Subalpine_Fir\">0.7</mo_pmpgVal>");
    oOut.write("</mo_prnciMaxPotentialSurvival>");
    oOut.write("<mo_prnciMaxCrowdingRadius>");
    oOut.write("<mo_pmcrVal species=\"Western_Hemlock\">10.0</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Western_Redcedar\">10.0</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Amabilis_Fir\">10.0</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Subalpine_Fir\">10.0</mo_pmcrVal>");
    oOut.write("</mo_prnciMaxCrowdingRadius>");
    oOut.write("<mo_prnciNeighDBHEff>");
    oOut.write("<mo_pndeVal species=\"Western_Hemlock\">2.17</mo_pndeVal>");
    oOut.write("<mo_pndeVal species=\"Western_Redcedar\">1.86</mo_pndeVal>");
    oOut.write("<mo_pndeVal species=\"Amabilis_Fir\">1.51</mo_pndeVal>");
    oOut.write("<mo_pndeVal species=\"Subalpine_Fir\">1.6</mo_pndeVal>");
    oOut.write("</mo_prnciNeighDBHEff>");
    oOut.write("<mo_prnciNeighDistEff>");
    oOut.write("<mo_pndseVal species=\"Western_Hemlock\">0.69</mo_pndseVal>");
    oOut.write("<mo_pndseVal species=\"Western_Redcedar\">0.46</mo_pndseVal>");
    oOut.write("<mo_pndseVal species=\"Amabilis_Fir\">0.18</mo_pndseVal>");
    oOut.write("<mo_pndseVal species=\"Subalpine_Fir\">0.28</mo_pndseVal>");
    oOut.write("</mo_prnciNeighDistEff>");
    oOut.write("<mo_prnciWestern_HemlockNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.66</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">0.37</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.40</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.43</mo_plVal>");
    oOut.write("</mo_prnciWestern_HemlockNeighborLambda>");
    oOut.write("<mo_prnciWestern_RedcedarNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.44</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">0.49</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">4.58E-4</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">5.58E-4</mo_plVal>");
    oOut.write("</mo_prnciWestern_RedcedarNeighborLambda>");
    oOut.write("<mo_prnciAmabilis_FirNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.41</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">6.21E-4</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.21</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.25</mo_plVal>");
    oOut.write("</mo_prnciAmabilis_FirNeighborLambda>");
    oOut.write("<mo_prnciSubalpine_FirNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.41</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">6.21E-4</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.21</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.18</mo_plVal>");
    oOut.write("</mo_prnciSubalpine_FirNeighborLambda>");
    oOut.write("<mo_prnciHybrid_SpruceNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.41</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">6.21E-4</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.27</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.22</mo_plVal>");
    oOut.write("</mo_prnciHybrid_SpruceNeighborLambda>");
    oOut.write("<mo_prnciLodgepole_PineNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.48</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">6.21</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.52</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.18</mo_plVal>");
    oOut.write("</mo_prnciLodgepole_PineNeighborLambda>");
    oOut.write("<mo_prnciTrembling_AspenNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.38</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">0.61</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.38</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.78</mo_plVal>");
    oOut.write("</mo_prnciTrembling_AspenNeighborLambda>");
    oOut.write("<mo_prnciBlack_CottonwoodNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.33</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">0.66</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.31</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.87</mo_plVal>");
    oOut.write("</mo_prnciBlack_CottonwoodNeighborLambda>");
    oOut.write("<mo_prnciPaper_BirchNeighborLambda>");
    oOut.write("<mo_plVal species=\"Western_Hemlock\">0.74</mo_plVal>");
    oOut.write("<mo_plVal species=\"Western_Redcedar\">0.19</mo_plVal>");
    oOut.write("<mo_plVal species=\"Amabilis_Fir\">0.14</mo_plVal>");
    oOut.write("<mo_plVal species=\"Subalpine_Fir\">0.88</mo_plVal>");
    oOut.write("</mo_prnciPaper_BirchNeighborLambda>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}
